package pmp;

/**
 * Contains names of configuration dorectives.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * <p>
 * Company:
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public final class ConfigDirectives {

    /**
     * Symbol marking start of block with macros. Default value: <code>""</code>.
     */
    public static final String CODE_START = "code.start";

    /**
     * Symbol marking start of block with macros. This token is handled
     * differently than others - empty string does not match. Default value:
     * <code>""</code>.
     */
    public static final String CODE_END = "code.end";

    /**
     * Controls handling with code marks. If <code>false</code>, marks are
     * discarded (default) else sent to output. Default value: <code>""</code>.
     */
    public static final String CODE_MARKS_OUTPUT = "code.marks.output";

    /**
     * Defines format of tokens. Parser treats tokens in the same way as single
     * characters that were not identified as special symbols (macronames,
     * string delimiters, etc).
     */
    public static final String PARSER_TOKEN = "code.token";

    /**
     * Regular expression that is used for looking for macro calls.
     * 
     * <p>
     * If you want to use prefix or/and postfix to be part of macro call but not
     * part of name of macro, you can use capturing groups. You can use more
     * groups, but only group 1 will be used as macro name.
     * </p>
     * <p>
     * <b>Note:</b> If macro name does not match this expression (resp.
     * capturing group 1), it cannot be called directly.
     * </p>
     */
    public static final String MACRO_PATTERN = "macro.pattern";

    /**
     * Fallback macro is called whenever parser comes across possible macro
     * name, but appropriate macro is not found.
     */
    public static final String MACRO_FALLBACK = "macro.fallback";

    /**
     * Specifies whether arguments must be present to call macro.
     * <p>
     * If set to <code>true</code>, macro name that is not followed by
     * opening bracket ({@link #MACRO_OPENING_BRACKET}) will not be recognized
     * as macro call and handled as text.
     * </p>
     * 
     */
    public static final String MACRO_ARGUMENTS_REQUIRED = "macro.args.required";

    /**
     * Controls whether macro calls are case-sensitive.
     * <p>
     * It only affects process of searching for macro in macro list (which is
     * always case-sensitive), and does not affect any other manipulations with
     * macro list (such as retriaval of configuration variable).
     * </p>
     * <p>
     * If this variable is empty (or not set at all), then parser tries to find
     * macro using case-insensitive search, if case-sensitive fails.
     * </p>
     * <p>
     * Default: "1"
     * </p>
     */
    public static final String MACRO_CASE_SENSITIVE = "macro.casesensitive";

    /** default: "\([ \t]*" */
    public static final String MACRO_OPENING_BRACKET = "macro.opening.bracket";

    /** default: "[ \t]*\)" */
    public static final String MACRO_CLOSING_BRACKET = "macro.closing.bracket";

    /** default: "[ \t]*,[ \t]" */
    public static final String MACRO_PARAM_SEPARATOR = "macro.paramseparator";


    /**
     * Encoder used by all <code>$(...)</code> constructions
     * in efined macros.
     * <p>   
     * default: undefined
     * </p>
     */
    public static final String DEFINED_ENCODER = "defined.encoder";

    /**
     * Separator used by <code>$(nnn-mmm)</code> constructions
     * in defined macros.  
     * <p>
     *  default: undefined
     * </p>
     */
    public static final String DEFINED_SEPARATOR = "defined.separator";

    
    /* STACK */
    /**
     * Number of items in array on top of te stack
     */
    public static final String STACK_ITEMS_COUNT = "stack.count";

    /**
     * Items of array on top of stack Meaning of items depends on invocation.
     * Numbering of items begins with 0 and ends with (STACK_ITEMS_COUNT-1).
     */
    public static final String STACK_ITEM__ = "stack.*";

    /** Symbol used as opening backet in current macro call. */
    public static final String STACK_OPENING_BRACKET = "stack.opening";

    /** Symbol used as opening backet in current macro call. */
    public static final String STACK_CLOSING_BRACKET = "stack.closing";

    /**
     * String processing mode. Affects the way the string is processed. Allowed
     * values:
     * <ul>
     * <li><em>copy</em> copies content of string to output (default)</li>
     * <li><em>discard</em> discards whole string as if it is comment</li>
     * <li><em>parse</em> processes special symbols and output result</li>
     * <li><em>nested</em> pares nested string </li>
     * </ul>
     * 
     * @see #VALUE_COPY
     * @see #VALUE_DISCARD
     * @see #VALUE_PARSE
     * @see #VALUE_NESTED
     */

    public static final String STRING__MODE = "string.*.mode";

    /**
     * String marks processing mode. Affects the way the string is processed.
     * Possible values:
     * <ul>
     * <li><em>copy</em> keeps marks with string</li>
     * <li><em>copy-start</em> keeps starting mark with string</li>
     * <li><em>copy-end</em> keeps terminating mark with string</li>
     * <li><em>discard</em> discard marks (default)</li>
     * </ul>
     */
    public static final String STRING__MARKS_MODE = "string.*.marks-mode";

    /**
     * Defines wheter this string can appear outside macro parameter list. If
     * set to true, this string can appear anywhere inside of code block.
     */
    public static final String STRING__STANDALONE = "string.*.standalone";

    /** Format of string tokens */
    public static final String STRING__TOKENS = "string.*.tokens";

    /**
     * String opening symbol. Must be defined. String types lookup is performed
     * using this template.
     */
    public static final String STRING__START = "string.*.start";

    /**
     * End-of-string mark. If empty or not set, it is considered to have same
     * value as corresponding STRING__START.
     */
    public static final String STRING__END = "string.*.end";

    /**
     * Format of special symbols inside string. These symbols wil be parsed
     * using associated parser for this string type. If sting mode is not
     * <code>parse</code> then this directive is ignored.
     * 
     * @see #STRING__MODE
     */
    public static final String STRING__SYMBOLS = "string.*.symbols";

    /**
     * Parser which handles special symbols. This directive contains parser
     * itself, not name of parser
     * 
     * @see #STRING__MODE
     */
    public static final String STRING__PARSER = "string.*.parser";

    /**
     * Output produced instead of string in discard mode.
     * @see #STRING__MODE
     */
    public static final String STRING__OUTPUT = "string.*.output";

    /**
     * Name of output file. This directive is provided by {@link InputOutput}.
     */
    public static final String OUTPUT = "output.name";

    /**
     * Encoding of input files. This directive is used by {@link InputOutput} to
     * determine encoding of source files
     * 
     * @see java.io.FileInputStream
     * @see java.io.InputStreamReader#InputStreamReader(java.io.InputStream,
     *      java.lang.String)
     */
    public static final String INPUT_ENCODING = "input.encoding";

    /**
     * Encoding of output file. This directive is provided by
     * {@link InputOutput}.
     */
    public static final String OUTPUT_ENCODING = "output.encoding";

    // public static final String INPUT_LINEFEEDS = "input.line-feeds";

    /** Code in this directive is automatically appended to program. */
    public static final String SOURCE_APPEND = "source.append";

    /** Code in this directive is automatically prepended before program. */
    public static final String SOURCE_PREPEND = "source.prepend";

    /** This directive contains last error message */
    public static final String LAST_ERROR = "last.error";

    /** This directive contains last warning message */
    public static final String LAST_WARNING = "last.warning";

    /** This directive contains last notice message */
    public static final String LAST_NOTICE = "last.notice";

    /**
     * Current locale as comma separated list for {@link java.util.Locale}
     * constructor.
     * <p>
     * Format of string is: <em>language + ["," country ["," variant]]</em>.
     * </p>
     */
    public static final String LOCALE = "locale";

    /** Current level of debug output */
    public static final String DEBUGLEVEL = "debug.level";

    /** Current prefix of configuration directives */
    public static final String PMP_PREFIX = "prefix";

    /**
     * Specifies string without special symbols.
     * 
     * @see #STRING__MODE
     * @see #STRING__MARKS_MODE
     */
    public static final String VALUE_COPY = "copy";

    /**
     * Starting symbol will be copied to output
     * 
     * @see #STRING__MARKS_MODE
     */
    public static final String VALUE_COPY_START = "copy-start";

    /**
     * Starting symbol will be copied to output
     * 
     * @see #STRING__MARKS_MODE
     */
    public static final String VALUE_COPY_END = "copy-end";

    /**
     * Specifies string which should be discarded.
     * 
     * @see #STRING__MODE
     * @see #STRING__MARKS_MODE
     */
    public static final String VALUE_DISCARD = "discard";

    /**
     * Specifies that string should be scanned for special symbols and then
     * outputted.
     * 
     * @see #STRING__MODE
     */
    public static final String VALUE_PARSE = "parse";

    /**
     * Specifies nested strings.
     * 
     * @see #STRING__MODE
     */
    public static final String VALUE_NESTED = "nested";

} // class ConfigDirectives
