package pmp;

/**
 * Represents empty regular expression matcher. Behaves exactly in the same way
 * as JavaMatcher object created with empty regular expression but this class is
 * much faster since it doesn't use regular expressions.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public final class EmptyMatcher implements MatcherIface {

    private final Token token;

    private boolean overfeed;

    /**
     * This is fast matcher for empty expression.
     */
    public static final String EMPTY_PATTERN = "";

    /**
     * Creates new instance.
     * 
     * @param token
     */
    public EmptyMatcher(Token token)
    {
        this.token = token;
        this.overfeed = false;
    }

    public void feed(int c)
    {
        overfeed = true;
    }

    public State getState()
    {
        if (overfeed) { return State.OVERFEED; }
        return State.MATCH;
    }

    public State matches(StringBuilder text)
    {
        if (text.length() > 0)
            return State.OVERFEED;
        else
            return State.MATCH;
    }

    public String getPattern()
    {
        return "";
    }

    public Token getToken()
    {
        return token;
    }

    public int getMatchingLength()
    {
        return 0;
    }

    @Override
    public String toString()
    {
        return "{ \"\", " + token.toString() + " }";
    }

    public void reset()
    {
        overfeed = false;
    }

}
