package pmp;

/**
 * Represents never-matching regular expression matcher.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public final class FailingMatcher implements MatcherIface {

    private final Token token;

    /**
     * Extension to Java regular expressions. 
     * This is invalid Java pattern <code>[]</code> which PMP
     * uses it to construct never-matching matcher.
     */
    public static final String FAILING_PATTERN = "[]";

    /**
     * Creates new instance.
     * 
     * @param token
     */
    public FailingMatcher(Token token)
    {
        this.token = token;
    }

    public void feed(int c)
    {
        // do nothing
    }

    public State getState()
    {
        return State.MISMATCH;
    }

    public State matches(StringBuilder text)
    {
        return State.MISMATCH;
    }

    public String getPattern()
    {
        return FAILING_PATTERN;
    }

    public Token getToken()
    {
        return token;
    }

    public int getMatchingLength()
    {
        assert false;
        return -1;
    }

    @Override
    public String toString()
    {
        return "{ \"" + FAILING_PATTERN + "\", " + token.toString() + " }";
    }

    public void reset()
    {
        // do nothing
    }

}
