package pmp;

/**
 * Interface for Macroprocessor input/output system.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class InputOutput extends Macroprocessor.Component {

 /**
  * Use <code>stdin</code> as input.
  */
   public static final String STD_INPUT = "pmp://stdin";

   /**
    * Use stdout as output.
    */
   public static final String STD_OUTPUT = "pmp://stdout";

   /**
    * Use stderr as output.
    */
   public static final String ERR_OUTPUT = "pmp://stderr";

   /**
    * Discard output.
    */
   public static final String NULL_OUTPUT = "pmp://null";
   /**
    * Use random generator as input.
    *  @todo not yet implemented 
    */
   public static final String RANDOM_INPUT = "pmp://random";

   /**
    * Use empty input.
    */
   public static final String NULL_INPUT = NULL_OUTPUT;

   public InputOutput(Macroprocessor mp)
   {
      mp.super();
   }

   /**
    * Returns default input source. May return <code>null</code>. In that
    * case user must fill InputQueue of macroprocessor manually.
    * 
    * @return default input source.
    */
   public InputSource getDefaultInput()
   {
       return null;
   }

   /**
    * Return new Input object for given source.
    * Must be reimplemented by subclasses.
    * 
    * @param name
    *           name of source
    * @return returns InputSource object for given source
    */
   public InputSource getInput(String name)
   {
       assert false;
       return null;
   }

   /**
    * Redirects output of macroprocessor to specified location.
    * Must be reimplemented by subclasses.
    * 
    * @param name
    *           name of output
    */
   public void setOutput(String name)
   {
       assert false;
   }

   /*
    * During one execution, this method should return always the same result
    * (ie: reference to one single instance of Writer)
    * 
    * @deprecated avoid using Reader/Writer @return Writer object
    */
   // public abstract Writer getOutputWriter();
   /**
    * Provides direct connection to output Writer
    * Must be reimplemented by subclasses.
    * 
    * @param str
    *           string to be written to output
    */
   public void write(String str)
   {
       assert false;
   }

   /**
    * This method should produce some kind of error message and then raise
    * {@link FatalParserException}.
    * 
    * @param error
    *           error message
    * @throws FatalParserException
    *            this exception should be thrown by all subclasses
    */
   public void errorHandler(String error) throws FatalParserException
   {
      // empty
   }

   public void warningHandler(String warning)
   {
      // empty
   }

   public void noticeHandler(String noticemessage)
   {
      // empty
   }

   public void debugHandler(String debugmessage)
   {
      // empty
   }

}
