package pmp;

import java.io.Closeable;

/**
 * Represents abstract input source used by macroprocessor's
 * InputQueue.
 * <p><b>Note</b>:
 *  You can get naming conflict with class org.xml.sax.InputSource. </p>
 * <p>Title: PMP: Macroprocessor</p>
 * <p>Description: Java macroprocessor</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public interface InputSource extends Closeable {
  /** 
   * Reads up to <code>length</code> bytes from input.
   * If end of source was reached (end of file, socket closed or something like that)
   * returns empty string.
   * @param maxLength maximal length of result; must not be zero!
   * @return returned string.
   */
  public String read(int maxLength);

  /** 
   * Closes source.
   * Macroprocessor will not call <code>read</code> after 
   * calling this method.
   */
  public void close();

}
