package pmp;

/**
 * Signals internal error in macroprocessor.
 * <p>
 * Error codes represent internal errors. Lower 16 bits contain BCD encoded
 * number of line (in time of writing; due to later changes in source file, real
 * line number might differ) where error occured and higher 16 bits identify
 * class where it happend.
 * </p>
 * <p>
 * See enum {@link pmp.InternalMPError.Source} for identificators of source.
 * </p>
 * <p>
 * For example: error code 0x020423 means that error occurred in
 * <code>MacroList</code> class at line number 423 (decimal).
 * </p>
 * <p>
 * Note: This class was rendered almost useless by <code>assert</code> feature
 * and further usage is deprecated.
 * </p>
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class InternalMPError extends Error {

    public enum Source {
        ANALYZER(0x01),
        MACROLIST(0x02),
        REGEXPS(0x03),
        MACROPROCESOR(0x04),
        INPUTQUEUE(0x05),
        CONFIGDIRECTIVES(0x06),
        ABSTRACTMACRO(0x07),
        MACROLISTITEM(0x08),
        GUI_MAINPANEL(0x82),
        GUI_GUITOOLS(0x81),
        REGEX_NODEPOINTER(0xA1),
        REGEX_PATTERNCOMPILER(0xA2);

        Source(int src)
        {
            this.src = src;
        }

        private final int src;

        int getSource()
        {
            return src;
        }
    }

    /** Error message used by {@link #InternalMPError(int)} constructor. */
    public static final String ERROR_CODE_MESSAGE = "Fatal parser error. Code = 0x";

    /**
     * Creates error with specified error code.
     * <p>
     * This constructor should be used when internal error occurs and there is
     * not any meaningfull message to be presented to user.
     * </p>
     * 
     * @param code
     *            error code (see description of this class for details)
     */

    public InternalMPError(int code)
    {
        super(ERROR_CODE_MESSAGE + Integer.toHexString(code));
    }

    public InternalMPError(Source file, int line)
    {
        this((file.getSource() << 16) | line);
    }

    public InternalMPError(int file, int line)
    {
        this((file << 16) | line);
    }

    private static final long serialVersionUID = -1L;

}
