package pmp;

import java.util.regex.*;

/**
 * Represents regular expression analyzer.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class JavaMatcher implements MatcherIface {

    /**
     * maximal length of buffer.
     * 
     * @todo this should be removed (how?)
     */
    public static final int MAX_TOKEN_LENGTH = 64;

    private State state;

    private boolean stateValid = false;

    private final Token token;

    private final String pattern;

    private final StringBuilder text;

    private final Matcher m;

    public JavaMatcher(Token token, Pattern pattern)
    {
        this.token = token;
        this.text = new StringBuilder(MAX_TOKEN_LENGTH);

        this.pattern = pattern.pattern();
        this.m = pattern.matcher("");

        matches();
    }

    @Override
    public String toString()
    {
        StringBuilder result = new StringBuilder(128);
        result.append("{ \"").append(pattern).append("\", ");
        result.append(token.toString()).append(" }");
        return result.toString();
    }

    public void reset()
    {
        text.setLength(0);
        stateValid = false;
    }

    public State getState()
    {
        if (!stateValid) matches();

        return state;
    }

    public String getPattern()
    {
        return pattern;
    }

    public Token getToken()
    {
        return token;
    }

    public int getMatchingLength()
    {
        if (!stateValid) {
            matches();
        }

        if (state == State.MATCH) {
            return text.length();
        } // if
        else if (state == State.OVERFEED) {
            m.lookingAt();
            return m.end();
        }
        return 0;
    }

    public void feed(int c)
    {

        // TODO: unicode support
        // text.append((char)c);
        stateValid = false;
    }

    public State matches(StringBuilder t)
    {
        this.text.setLength(0);
        this.text.append(t);
        matches();
        return state;
    }

    private void matches()
    {
        m.reset(text);

        if (m.matches()) {
            state = State.MATCH;
        } // if
        else if (m.lookingAt()) {
            state = State.OVERFEED;
        }
        else {
            if (text.length() >= MAX_TOKEN_LENGTH) {
                state = State.MISMATCH;
            }
            else {
                state = State.PARTIAL;
            }
        } // if/else

        stateValid = true;

    }

}
