package pmp;

/**
 * Defines macro with assigned output encoder.
 * <p>
 * This class encapsulates two macros. <code>macro</code> is invoked using
 * given arguments and <code>outputEncoder</code> is called with output from
 * <code>macro</code> as first parameter. Result of <code>outputEncoder</code>
 * is returned.
 * </p>
 * <p>
 * This class is used by <code>builtin</code> when called with three
 * parameters
 * </p>
 * 
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public final class MacroChain extends AbstractMacro {

   private final AbstractMacro macro1;
   private final AbstractMacro macro2;

   public MacroChain(AbstractMacro macro, AbstractMacro outputEncoder)
   {
      this.macro1 = macro;
      this.macro2 = outputEncoder;
      if ((macro1 == null) || (macro2 == null)) {
         throw new FatalParserException(
               "Internal error: NullPointerException in MacroChain class.");
      } // if

      // modify inherited field
      super.PARAMS_REQUIRED = macro1.getRequiredParams();
   }

   public AbstractMacro getMacro()
   {
      return macro1;
   }

   public AbstractMacro getOutputEncoder()
   {
      return macro2;
   }

   @Override
   public String run(Macroprocessor mp, String[] args)
   {

      String m1result = macro1.run(mp, args);

      String[] args2 = { args[0], m1result };

      return macro2.run(mp, args2);

   }

}
