package pmp;

/**
 * Interface for regexp matcher class.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * <p>
 * Company:
 * </p>
 * 
 * @author Luděk Hlaváček
 * @version 1.0
 */

public interface MatcherIface {

	/**
	 * State of matcher.
     * @author Luděk Hlaváček
	 *
	 */
	public enum State {
		/**
		 * Input string does not match the pattern.
		 */
		MISMATCH,
		/**
		 * Input string matches the beginning of the pattern. After processing
		 * more characters from input sequence, state may change to
		 * {@link #MISMATCH} or {@link #MATCH}.
		 */
		PARTIAL,
		/**
		 * Beginning of input sequence matches the pattern. (method
		 * <code>lookingAt()</code> of standard Java Matcher class would
		 * return <code>true</code>.
		 */
		OVERFEED,

		/**
		 * Entire input sequence matches the pattern. (method
		 * <code>matches()</code> of standard Java Matcher class would return
		 * <code>true</code>.
		 */
		MATCH
	}

	/**
	 * Appends character to matched string snd updates stato of this matcher.
	 * 
	 * @param c
	 *            chracter to be appended
	 */
	public void feed(int c);

	
      /**
       * Resets this matcher with given string.
       * @return new state of matcher
       */
	public State matches(StringBuilder text);

	/**
	 * Resets matcher.
	 */
	public void reset();

	/**
	 * Returns token that would be produced by this class.
	 * 
	 * @return token object
	 */
	public Token getToken();

	/**
	 * Returns length of input subsequence that matches pattern.
	 * Valid only for states {@link State#PARTIAL} and {@link State#OVERFEED}.
	 * @return number of characters matching the pattern
	 */
	public int getMatchingLength();

	/**
	 * Returns state of matcher.
	 * 
	 * @return result of matching procedure
	 */
	public State getState();

    /**
     * Returns pattern associated with this matcher.
     * @return regular expression 
     */
    public String getPattern();

}
