/* 
 * Copyright  2006 Luděk Hlaváček
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */ 

package pmp.ant;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import static pmp.xml.XMLTools.*;

/**
 * Class representing <code>config</code> subelement of
 * PMP ant task.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class ConfigElement {

    private enum ElementType {
        CLASS(ELEMENT_CLASS),
        DEFINE(ELEMENT_DEFINE),
        LIBRARY(ELEMENT_LIBRARY),
        IMPORT(ELEMENT_IMPORT),
        UNKNOWN("INVALID");

        private ElementType(String tag)
        {
            this.tagName = tag;
        }

        private String tagName;

        protected String getTagName()
        {
            return tagName;
        }
    }

    public static class ConfigSubElement {

        /**
         * Attribute name.
         */
        protected String name = null;

        /**
         * Attribute readOnly.
         */
        protected Boolean readOnly = null;

        /**
         * Attribute outputEncoder.
         */
        protected String outputEncoder = null;

        /**
         * Attribute format.
         */
        protected String format = null;

        /**
         * Text node of this element.
         */
        protected String data = null;

        /**
         * Type of element (tagName).
         */
        protected ElementType type = ElementType.UNKNOWN;

        public void addText(String textdata)
        {
            this.data = textdata;
        }

        public void setName(String name)
        {
            this.name = name;
        }

        public void setFormat(String format)
        {
            this.format = format;
        }

        public void setoutputEncoder(String outputEncoder)
        {
            this.outputEncoder = outputEncoder;
        }

        public void setReadOnly(boolean ro)
        {
            this.readOnly = Boolean.valueOf(ro);
        }

    }// ConfigSubElement

    protected List<ConfigSubElement> items = new ArrayList<ConfigSubElement>(32);

    protected String code = null;

    protected Document xmldoc;

    protected Element root;

    protected ConfigElement() 
    {
        // nothing
    }

    public ConfigSubElement createClass()
    {
        ConfigSubElement cfg = new ConfigSubElement();
        cfg.type = ElementType.CLASS;
        items.add(cfg);
        return cfg;
    }

    public ConfigSubElement createDefine()
    {
        ConfigSubElement cfg = new ConfigSubElement();
        cfg.type = ElementType.DEFINE;
        items.add(cfg);
        return cfg;
    }

    public ConfigSubElement createLibrary()
    {
        ConfigSubElement cfg = new ConfigSubElement();
        cfg.type = ElementType.LIBRARY;
        items.add(cfg);
        return cfg;
    }

    public ConfigSubElement createImport()
    {
        ConfigSubElement cfg = new ConfigSubElement();
        cfg.type = ElementType.IMPORT;
        items.add(cfg);
        return cfg;
    }

    protected void build() throws ParserConfigurationException
    {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(false);
        factory.setCoalescing(true);
        factory.setIgnoringComments(true);
        factory.setNamespaceAware(false);
        DocumentBuilder builder = factory.newDocumentBuilder();
        xmldoc = builder.newDocument();
        xmldoc.appendChild(xmldoc.createElement(ELEMENT_ROOT));
        root = xmldoc.getDocumentElement();

        for (Iterator<ConfigSubElement> i = items.iterator(); i.hasNext();) {
            ConfigSubElement cfg = i.next();
            Element el = xmldoc.createElement(cfg.type.getTagName());

            if (cfg.name != null) el.setAttribute(ATTR_NAME, cfg.name);
            if (cfg.readOnly != null)
                el.setAttribute(ATTR_READONLY, cfg.readOnly.toString());
            if (cfg.outputEncoder != null)
                el.setAttribute(ATTR_OUTPUT_ENCODER, cfg.outputEncoder);
            if (cfg.format != null) el.setAttribute(ATTR_FORMAT, cfg.format);

            if (cfg.data != null)
                el.appendChild(xmldoc.createTextNode(cfg.data));

            root.appendChild(el);
        }// for
    }

}
