/* 
 * Copyright  2006 Luděk Hlaváček
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */ 


package pmp.ant;

import java.util.List;
import java.util.ArrayList;

import org.apache.tools.ant.BuildException;

/**
 * Class representing <code>export</code> subelement of PMP ant task.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class ExportElement {

    enum Modes { FIRST, LAST, CONCAT, SUM }

    protected static final int TYPE_MACRO = 0x701;

    protected static final int TYPE_CLASSNAME = 0x702;

    protected static final int TYPE_ALL = 0x703;

    public final List<ExportItem> exports = new ArrayList<ExportItem>(16);

    public static class ExportItem {

        /** Attribute name (name of directive). */
        protected String name = null;

        /** Text node of this element (name of macro). */
        protected String macro = "";

        /** Exported value. */
        protected String value = null;

        /** Attribute mode. */
        protected Modes mode = Modes.FIRST;

        /** Attribute type. */
        protected int type = 0;

        protected ExportItem(int type)
        {
            this.type = type;
        }

        public void addText(String textdata)
        {
            this.macro = textdata;
            if (this.name == null)
                this.name = textdata;
        }

        public void setName(String name)
        {
            if (type == TYPE_ALL) { 
                throw new BuildException(
              "The name attribute is not supported in \"all\" element."); 
            }
            this.name = name;
        }

        public void setMode(String mode)
        {
            try {
            this.mode = Modes.valueOf(mode.toUpperCase());
            } catch (IllegalArgumentException ex) {
                throw new BuildException("Invalid value for mode attribute: " + mode);
            }
        }

    }// class ExportItem

    protected ExportElement()
    {
        // nothing
    }

    public ExportItem createAll()
    {
        ExportItem ex = new ExportItem(TYPE_ALL);
        exports.add(ex);
        return ex;
    }

    public ExportItem createMacro()
    {
        ExportItem ex = new ExportItem(TYPE_MACRO);
        exports.add(ex);
        return ex;
    }

    public ExportItem createClassName()
    {
        ExportItem ex = new ExportItem(TYPE_CLASSNAME);
        exports.add(ex);
        return ex;
    }

}
