package pmp.cli;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

import pmp.Version;

/**
 * Default main class. Checks parameters for
 * <code>"-gui"</code> <code>"--help"</code> and <code>"-version"</code>
 * and invokes appropriate user interface.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class Launcher {

    /**
     * Location of short help with syntax.
     */
    public static final String HELP_FILE = "/pmp/help/syntax.txt";

    /** Displays version */
    public static final String OPTION_VERSION = "-version";

    /** Display help for syntax and options. */
    public static final String OPTION_HELP = "-help";

    /** Run GUI version */
    public static final String OPTION_GUI = "-gui";

    /** Displays version (short option) */
    public static final String SHORT_OPTION_VERSION = "-v";

    /** Display help for syntax and options (short option) */
    public static final String SHORT_OPTION_HELP = "-h";

    /** Run GUI version (short option) */
    public static final String SHORT_OPTION_GUI = "-g";

    /** Display help for syntax and options (alternative syntax) */
    public static final String OPTION_HELP_QMARK = "-?";

    /**
     * Main method of macroprocessor. Determines using command line arguments
     * what to do.
     * 
     * @param args
     *            command line arguments
     * @throws Exception
     */
    public static void main(String[] args) throws Exception
    {
        String arg0 = "";
        if (args[0] != null) 
            arg0 = args[0].toLowerCase();

        if (args.length == 0) {
            pmp.cli.Main.main(args);
        }
        else if ((OPTION_HELP.equals(arg0)) || (SHORT_OPTION_HELP.equals(arg0))
                || (OPTION_HELP_QMARK.equals(arg0))) {
            printHelp();
        }
        else if (OPTION_VERSION.equals(arg0)
                || SHORT_OPTION_VERSION.equals(arg0)) {
            printVersion();
        }
        else if (OPTION_GUI.equals(arg0) || SHORT_OPTION_GUI.equals(arg0)) {
            pmp.gui.MainWindow.main(args);
        }
        else {
            pmp.cli.Main.main(args);
        }
    }

    static void printVersion()
    {
        System.err.println(Version.PROJECT_NAME + " \"" + Version.VERSION
                + "\"");
    }

    static void printHelp() throws IOException
    {
        Reader in = new InputStreamReader(Launcher.class
                .getResourceAsStream(HELP_FILE), "UTF-8");
        int res;
        char[] buffer = new char[256];
        while ((res = in.read(buffer, 0, 256)) != -1) {
            System.err.print(new String(buffer, 0, res));
        } // while
        in.close();
    }
}
