package pmp.gui;

import java.io.IOException;
import java.net.URL;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;

import pmp.Version;

/**
 * "About" dialog.
 * <p>Title: PMP: Macroprocessor</p>
 * <p>Description: Java macroprocessor</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * @author Luděk Hlaváček
 * @version 1.0
 */

public class AboutFrame {

  public static final String ABOUT_INFO =
      Version.PROJECT_NAME + " " + Version.VERSION + "\n" +
      "Copyright \u00A9 2005 Luděk Hlaváček";

  /**
   * Shows About dialog
   * @param parent parent component
   */
  public static void showAboutFrame(Component parent)
  {
    JOptionPane.showMessageDialog(parent,
                                  ABOUT_INFO,
                                  "About...",
                                  JOptionPane.INFORMATION_MESSAGE,
                                  GUITools.getIcon("about"));
  }

  /** Help frame   */
  protected static JFrame frame = null;

  /**
   * Shows Help frame
   */
  public static void showHelpFrame()
  {
    if ( (frame != null)) {
      frame.dispose();
    }
    frame = new JFrame(Const.TITLE_HELP);
    frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    Container pane = frame.getContentPane();

    pane.setLayout(new BorderLayout()); // create editor pane
    pane.add(new JScrollPane(createEditorPane()), BorderLayout.CENTER);

    JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER)); // create button
    buttonPanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
    JButton button = new JButton(Const.BUTTON_OK);
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent)
      {
        frame.dispose();
        frame = null;
      }
    });
    buttonPanel.add(button);
    pane.add(buttonPanel, BorderLayout.SOUTH);

    frame.pack();
    GUITools.centreWindow(frame);
    frame.setVisible(true);
  }

  /** Location of help file for GUI.  */
  public static final String HELP_FILE = "/pmp/help/gui.html";

  /**
   * Creates {@link java.swing.JEditorPane} HTML viewer fror
   * displaying help.
   * @return newly created component
   */
  private static JEditorPane createEditorPane()
  {

    JEditorPane editorPane = new JEditorPane();
    editorPane.setEditable(false);

    URL helpURL = AboutFrame.class.getResource(HELP_FILE);
    if (helpURL != null) {
      try {
        editorPane.setPage(helpURL);
      }
      catch (IOException e) {
        editorPane.setText("Attempted to read a bad URL: " + helpURL);
      }
    }
    else {
      editorPane.setText("Couldn't find file: " + HELP_FILE);
    }
    editorPane.setBorder(BorderFactory.createEmptyBorder(20, 10, 10, 10));
    editorPane.setPreferredSize(new Dimension(400, 300));
    return editorPane;
  }

}
