package pmp.gui;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import java.awt.*;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.JTextComponent;

import pmp.InternalMPError;

/**
 * Tools for GUI.
 * <p>
 * Title: Mail client
 * </p>
 * <p>
 * Description: Mail client
 * </p>
 * <p>
 * Copyright: Copyright (c) 2004, LH
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public final class GUITools {

    private GUITools()
    {
        // hide constructor
    }

    /** Empty icon. Use as a replacement for nonexisting icons. */
    public static final String ICON_EMPTY = "empty";

    public static final String ICON_CLOSE = "close";

    public static final String ICON_EXIT = "exit";

    public static final String ICON_MACROS = "macros";

    public static final String ICON_LOADXML = "loadxml";

    public static final String ICON_LOADDEF = "loaddef";

    public static final String ICON_ABOUT = "about";

    public static final String ICON_RUN = "run";

    public static final String ICON_HELP = "help";

    /** Cache for icons loaded from disk. */
    protected static Map<String, ImageIcon> icons;

    /** Static initialization. */
    static {
        icons = new HashMap<String, ImageIcon>(32);
        getIcon(ICON_EMPTY); // load empty icon
    }

    /**
     * Looks for icon in cache or loads image from file using
     * {@link #loadIcon(String)}.
     * 
     * @param filename
     *            name of PNG file without extension and path.
     * @return loaded image or null.
     */
    public static ImageIcon getIcon(String filename)
    {
        if (icons.containsKey(filename)) {
            return icons.get(filename);

        }
        else {
            ImageIcon newIcon = loadIcon(filename);
            if (newIcon == null) {
                newIcon = icons.get(ICON_EMPTY);
            } // if
            icons.put(filename, newIcon);
            return newIcon;

        } // if/else
    }

    /**
     * Loads image from file.
     * 
     * @param filename
     *            name of PNG file without extension and path.
     * @return loaded image or null.
     */
    protected static ImageIcon loadIcon(String filename)
    {
        ImageIcon icon = null;

        if ((filename != null) && (filename.length() > 0)) {

            URL url = GUITools.class.getResource("/pmp/icons/" + filename
                    + ".png");
            // load icon: ${packagename}/icons/${filename}.png
            if (url != null) {
                icon = new ImageIcon(url);
            }
        } // if
        return icon;
    }

    /**
     * Sets window's position to screen centre.
     * 
     * @param frame
     *            window to move to the centre of screen
     */
    public static void centreWindow(Window frame)
    {
        Dimension dimScreen = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension dimFrame = frame.getSize();
        frame.setLocation((dimScreen.width - dimFrame.width) / 2,
                (dimScreen.height - dimFrame.height) / 2);
    }

    /**
     * Returns {@link javax.swing.filechooser.FileFilter} for selecting XML
     * files.
     */
    public static FileFilter getXmlFilter()
    {
        return new FileFilter() {

            @Override
            public boolean accept(java.io.File f)
            {
                return f.getName().endsWith(".xml");
            }

            @Override
            public String getDescription()
            {
                return "XML files";
            }
        };
    }

    /**
     * Tries to obtain String value from JComponent. If fails returns empty
     * string <code>""</code> (not <code>null</code>)
     * 
     * @param editor
     *            given component
     * @return obtained string
     */
    public static String getComponentValue(JComponent editor)
    {
        if (editor instanceof JPasswordField) {
            char[] pass = ((JPasswordField) editor).getPassword();
            String password = new String(pass);
            java.util.Arrays.fill(pass, '\0');
            return password;
        }
        if (editor instanceof JTextComponent) { return ((JTextComponent) editor)
                .getText(); }
        if (editor instanceof JComboBox) { return ((JComboBox) editor)
                .getSelectedItem().toString(); }
        if (editor instanceof JLabel) { return ((JLabel) editor).getText(); }
        if (editor instanceof AbstractButton) { return ((AbstractButton) editor)
                .getActionCommand(); }
        throw new InternalMPError(0x810130);
    }

}
