package pmp.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import static pmp.gui.Const.*;

/**
 * Main class for GUI interface of PMP.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class MainWindow implements Runnable, ActionListener {

    private JFrame mainWindow;

    public MainWindow()
    {
        // empty
    }

    public static void main(String[] args)
    {
        SwingUtilities.invokeLater(new MainWindow());
    }

    /**
     * Method specified by {@link Runnable} interface. Invoked by AWT thread.
     */
    public void run()
    {
        createAndShowGUI();
    }

    /**
     * Creates GUI. Should be invoked from AWT dispatching thread.
     */
    public void createAndShowGUI()
    {
        mainWindow = new JFrame();

        mainWindow.setTitle(Const.TITLE_MAINFRAME);
        mainWindow.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        MainPanel pmpPanel = new MainPanel(mainWindow, this);

        pmpPanel.setOpaque(true); // content panes must be opaque
        mainWindow.setContentPane(pmpPanel);

        mainWindow.setJMenuBar(pmpPanel.createMenu());

        mainWindow.pack();
        GUITools.centreWindow(mainWindow);
        mainWindow.setVisible(true);
    }

    public void actionPerformed(ActionEvent e)
    {
        String action = e.getActionCommand();

        if (MENU_QUIT.equals(action)) {
            mainWindow.dispose();
        } // MENU_QUIT

        else if (MENU_ABOUT.equals(action)) {
            AboutFrame.showAboutFrame(mainWindow);
        } // MENU_ABOUT

        else if (MENU_HELP.equals(action)) {
            AboutFrame.showHelpFrame();
        } // MENU_HELP

    }

}
