package pmp.io;

import pmp.InputSource;
import java.util.Random;

/**
 * This class allows to give random data as input to macroprocessor.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class RandomInputSource implements InputSource {

   public String read(int maxLength)
   {
      if (length == 0)
         return "";

      if ((length > 0) && (maxLength > length))
         maxLength = length;

      length -= maxLength;

      char[] buffer = new char[maxLength];
      for (int i = 0; i < maxLength; i++)
         buffer[i] = (char) random.nextInt(0x10000);
      return String.valueOf(buffer);
   }

   public void close()
   {
      // do nothing
   }

   private final Random random;

   private int length;

   public RandomInputSource(Random source, int length)
   {
      if (source == null) {
         long seed = System.currentTimeMillis()
               ^ ((new Object()).hashCode() << 32)
               ^ Runtime.getRuntime().freeMemory();

         random = new java.util.Random(seed);
      } else {
         random = source;
      }
      this.length = length;
   }

} // RandomInputSource
