package pmp.macro;

import pmp.*;

/** 
 * Calls macro given in first parameters.
 * Remaining parameters are passed to invoked macro.
 * <p>Title: PMP: Macroprocessor</p>
 * <p>Description: Java macroprocessor</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class Call
    extends AbstractMacro {

  public static final int PARAMS_REQUIRED = 1;

  /**
   * Load macro from class file.
   */
  public static final String PREFIX_JAVA = "java:";
  /**
   * Load macro from macro namespace.
   */
  public static final String PREFIX_PMP = "pmp:";

  @Override
  public String run(Macroprocessor mp, String[] params)
  {

    AbstractMacro macro = null;
    String[] macroParams = new String[params.length - 1];
    System.arraycopy(params, 1, macroParams, 0, params.length - 1);

    if (params[1].startsWith(PREFIX_JAVA)) {
      String macroClass = params[1].substring(PREFIX_JAVA.length());
      macro = mp.loadFromClass(macroClass);
    } //if

    else if (params[1].startsWith(PREFIX_PMP)) {
      String macroClass = params[1].substring(PREFIX_PMP.length());
      macro = mp.getMPMacros().getMacro(macroClass);
    } //if

    else {
      macro = mp.getMPMacros().getMacro(params[1]);
    }

    if (macro != null) {
      return mp.runMacro(
          new CallStackItem(CallStackItem.Type.MACRO, macroParams, macro));
    }

    else {
      mp.warning("Undefined macro: " + params[1]);
      return "";
    }
  }

}
