package pmp.macro;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/** 
 * Prints string directly into macroprocessor output.
 * Use with care. If invoked from inside other mocro's parameters,
 * given string will be ouputted before any other output from
 * enclosing macro.
 * <p>
 * <code>ifelse('abc','.',echo('hello'))</code>
 * will output "hello.".</p>
 * <p>
 * Processing sequence:<br />
 * <code>ifelse('abc','.',echo('hello'))</code><br />
 * <code>ifelse('abc','.','')</code> => outputs 'hello'<br />
 * <code>'.'</code> => outputs '.'<br />
 * </p>
 * <p>Title: PMP: Macroprocessor</p>
 * <p>Description: Java macroprocessor</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class Echo
    extends AbstractMacro {

  public static final int PARAMS_REQUIRED = 1;

  @Override
  public String run(Macroprocessor mp, String[] params)
  {
    mp.getMPIO().write(params[1]);
    return "";
  }

}
