package pmp.macro;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * <p>
 * Halts macroprocessor.
 * </p>
 * <p>
 * First parameter specifies type of message (passed in second parameter)
 * produced before halting.
 * </p>
 * Possible values are <code>ERROR</code>, <code>WARNING</CODE>,
 * <code>NOTICE</code> and <code>SILENT</code>. 
 * <code>ERROR</code> means that FatalParserException exception will be thrown.
 * <code>SILENT</code> will not produce any output. 
 * <p>Note: For now, this macro just flushes macroprocessor's input queue.
 * Due to this, macroprocessor may throw "Unexpected end of input" exception
 * in some cases. </p>
 * 
 * <p>Title: PMP: Macroprocessor</p>
 * <p>Description: Java macroprocessor</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 * @see pmp.FatalParserException
 */

public class Halt extends AbstractMacro {

   public static final int PARAMS_REQUIRED = 2;

   public static final String HALT_ERROR = "ERROR";

   public static final String HALT_WARNING = "WARNING";

   public static final String HALT_NOTICE = "NOTICE";

   public static final String HALT_SILENT = "SILENT";

   @Override
   public String run(Macroprocessor mp, String[] args)
   {
      String arg1 = args[1].trim().toUpperCase();

      if (arg1.equals(HALT_ERROR)) {
         mp.error(args[2]);
      } else if (arg1.equals(HALT_WARNING)) {
         mp.warning(args[2]);

      } else if (arg1.equals(HALT_NOTICE)) {
         mp.notice(args[2]);
      } else if (arg1.equals(HALT_SILENT)) {
         // nothing
      }

      mp.getMPIQ().reset();

      return "";
   }
}
