package pmp.macro;

import pmp.*;

/** 
 * Returns <code>"1"</code> if macro with name specified by first parameter
 * exists and is defined (not builtin).
 * Type of {@link MacroChain} is considered to be same as first
 * encapsulated macro.
 * <p>
 * If more than one parametr is given,
 * then second parametr is returned instead of<code>1</code>
 * and third parametr is returned otherwise
 * (provided the third parameter is specified).
 * </p>
 * <p>Title: PMP: Macroprocessor</p>
 * <p>Description: Java macroprocessor</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class IsDefined
    extends AbstractMacro {

  public static final int PARAMS_REQUIRED = 1;

  @Override
  public String run(Macroprocessor mp, String[] args)
  {
    AbstractMacro macro = mp.getMPMacros().getMacro(args[1]);

    String yes = (args.length > 2) ? args[2]: "1";
    String no = (args.length > 3) ? args[3]: "";
    
    if (macro == null) { // exists?
      return no;
    }

    if (macro instanceof DefinedMacro) { // is defined ?
      return yes;
    }

    if ( (macro instanceof MacroChain) // is MacroChain ?
        && ( ( (MacroChain) macro).getMacro()instanceof DefinedMacro)) {
      return yes;
    }

    return no;
  }
}