package pmp.macro.java;

import java.lang.reflect.*;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Creates new instance of given class. The first parameter contains name of
 * class. New object is instantied using constuctor which accepts remaining
 * parameters of this macro as its arguments.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 * @see pmp.macro.java.ObjectManager
 */

public class Construct extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 1;

    @Override
    public String run(Macroprocessor mp, String[] args)
    {

        final String className = args[1].trim();
        final ObjectManager mgr = ObjectManager.getInstance();

        if (className.length() == 0) {
            mp.warning("Macro Construct: Missing class name.");
            return "";
        }

        try {
            final Class c = Class.forName(className);
            final Class[] expParams = mgr.getClasses(args, 2);
            final Constructor[] clist = c.getConstructors();
            Constructor constr = null;

            for (Constructor cc : clist) {

                final Class<?>[] params = cc.getParameterTypes();

                if (ObjectManager.testCompatibility(params, expParams)) {
                    constr = cc;
                    break;
                }// if

            }// for

            if (constr == null)
                throw new NoSuchMethodException("Constructor not found: "
                        + args[1]);

            final Object result = constr.newInstance(mgr.getObjects(args, 2));
            return mgr.addObject(result);

        } // try
        catch (InvocationTargetException ex) {
            mp.error("Macro Construct: Couldn't construct instance of "
                    + className);
        } catch (NoSuchMethodException ex) {
            mp
                    .error("Macro Construct: Couldn't find appropriate constructor for "
                            + className);
        } catch (IllegalAccessException ex) {
            mp.error("Macro Construct: Couldn't create instance of "
                    + className);
        } catch (InstantiationException ex) {
            mp.error("Macro Construct: Couldn't create instance of "
                    + className);
        } catch (ClassNotFoundException ex) {
            mp.error("Macro Construct: Couldn't load class: " + className);
        } catch (IllegalArgumentException ex) {
            mp.error("Macro Invoke: Invalid identifier in parameters for constructor  " + args[1]);
        } catch (IndexOutOfBoundsException ex) {
            mp.error("Macro Invoke: Invalid identifier in parameters for constructor  " + args[1]);
        }
        return "";
    }

}
