package pmp.macro.java;

import java.lang.reflect.*;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Returns value of field as object index. 
 * The first parameter contains object
 * identifier, the second name of field.
 * The Third parameter may contain new value.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 * 
 */

public class Field extends AbstractMacro {

   public static final int PARAMS_REQUIRED = 1;

   @Override
   public String run(Macroprocessor mp, String[] args)
   {
      ObjectManager objmgr = ObjectManager.getInstance();
      Object obj = null;
      Class c;
      boolean fstatic = false;

      try {
         // must be static ?
         if (args[1].indexOf(ObjectManager.OBJ_ID_SEPARATOR) == -1) {
            fstatic = true;
            c = Class.forName(args[1]);
         } else {
            obj = objmgr.getObject(args[1]);
            c = obj.getClass();
         }

         java.lang.reflect.Field f = c.getField(args[2]);
         if (fstatic && (!Modifier.isStatic(f.getModifiers()))) { 
            mp.error("Macro Field: Field must be static: " + args[2]);
            return "";
         }

         Object val = f.get(obj);
         if (args.length == 4) {
            f.set(obj, objmgr.getObject(args[3]));
         }
         return objmgr.addObject(val);
         
      } catch (IllegalArgumentException ex) {
          mp.error("Macro Field: invalid object identifier " + args[1]);
      } catch (IndexOutOfBoundsException ex) {
         mp.error("Macro Field: invalid identifier " + args[1]);
      } catch (ClassNotFoundException ex) {
         mp.error("Macro Field: Class not found - " + args[1]);
      } catch (NoSuchFieldException ex) {
         mp.error("Macro Field: Field not found - " + args[2]);
      } catch (IllegalAccessException ex) {
         mp.error("Macro Field: Cannot access field - " + args[2]);
      }
      return "";
   }

}
