package pmp.macro.java;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Returns <code>"1"</code> if given object is instance of specified class.                                       
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 * 
 */

public class InstanceOf extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 2;

    @Override
    public String run(Macroprocessor mp, String[] args)
    {
        ObjectManager mgr = ObjectManager.getInstance();
        String className = args[2].trim();
        try {
            Object obj = mgr.getObject(args[1]);
            Class<?> cl = Class.forName(className);
            if (obj == null) {
                Class<?> ocl = Class.forName(mgr.extractClassName(args[1]));
                if (cl.isAssignableFrom(ocl))
                    return "1";
                else
                    return "";
            }
            else if (cl.isInstance(obj)) {
                return "1";
            }
            else {
                return "";
            }

        } catch (ClassNotFoundException ex) {
            mp.error("Macro InstanceOf: Unknown class: " + className);
            return "";
        }
    }

}
