package pmp.macro.java;

import java.lang.reflect.*;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Invokes method of specified object. The first parametr specifies name of
 * class or object identifier, the second contains name of method. Remaining parametrs contain
 * argumets passed to this method.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class Invoke extends AbstractMacro {

   public static final int PARAMS_REQUIRED = 2;

   @Override
   public String run(Macroprocessor mp, String[] args)
   {
      final ObjectManager objmgr = ObjectManager.getInstance();
      Object obj = null;
      boolean fstatic = false;
      Method[] mlist;
      Class c;
      try {
         // must be static ?
         if (args[1].indexOf(ObjectManager.OBJ_ID_SEPARATOR) == -1) {
            fstatic = true;
            c = Class.forName(args[1]);
         } else {
            obj = objmgr.getObject(args[1]);
            c = Class.forName(objmgr.extractClassName(args[1]));
         }

         mlist = c.getMethods();
         final Class[] expParams = objmgr.getClasses(args, 3);
         Method m = null;

         for (Method mc : mlist) {

            if (!mc.getName().equals(args[2]))
               continue;
            if (fstatic && !Modifier.isStatic(mc.getModifiers()))
               continue;

            final Class<?>[] params = mc.getParameterTypes();
            if (ObjectManager.testCompatibility(params, expParams)) {
               m = mc;
               break;
            }//if

         }// for

         if (m == null)
            throw new NoSuchMethodException("Macro invoke: Method not found: "
                  + args[2]);
         final Object result = m.invoke(obj, objmgr.getObjects(args, 3));
         return objmgr.addObject(result);

      } catch (IllegalArgumentException ex) {
          mp.error("Macro Invoke: Invalid identifier " + args[1]);
      } catch (IndexOutOfBoundsException ex) {
         mp.error("Macro Invoke: Invalid identifier " + args[1]);
      } catch (ClassNotFoundException ex) {
         mp.error("Macro Invoke: Class not found - " + args[1]);
      } catch (NoSuchMethodException ex) {
         mp.error("Macro Invoke: Method not found - " + args[2]);
      } catch (IllegalAccessException ex) {
         mp.error("Macro Invoke: Cannot access method - " + args[2]);
      } catch (InvocationTargetException ex) {
         mp.error("Macro Invoke: Cannot invoke method - " + args[2]);
      }

      return "";
   }
}
