package pmp.macro.java;

import java.lang.reflect.Array;
import java.util.regex.Matcher;

import pmp.AbstractMacro;
import pmp.Macroprocessor;
import pmp.RegExps;

/**
 * <p>
 * Creates new instance of for given class (parameter 1) with given contents
 * (parameter 2). Returns object identifier.
 * </p>
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 * @todo : rewrite to support arrays & primitive types
 */

public class New extends AbstractMacro {

   public static final int PARAMS_REQUIRED = 1;

   private static Character decodeChar(String ch)
   {
      if (ch.length() == 1)
         return Character.valueOf(ch.charAt(0));
      else
         return Character.valueOf((char) Short.decode(ch).shortValue());
   }

   private static boolean decodeBoolean(String b)
   {
      if ("TRUE".equalsIgnoreCase(b) || "YES".equalsIgnoreCase(b))
         return true;
      else if ("FALSE".equalsIgnoreCase(b) || "NO".equalsIgnoreCase(b))
         return false;
      else
         return (Long.decode(b).longValue() != 0);
   }

   @Override
   public java.lang.String run(Macroprocessor mp, java.lang.String[] args)
   {
      final ObjectManager mgr = ObjectManager.getInstance();

      Matcher m = RegExps.compilePattern(
            ObjectManager.OBJ_ID_CLASSNAME
                  + "(\\[([0-9]*|0x[0-9a-fA-F]+|0b[01]+)\\])?").matcher("");
      m.reset(args[1]);

      if (!m.matches()) {
         mp.error("Macro New: Invalid class name: " + args[1]);
         return "";
      }

      boolean array = (m.group(2) != null);

      if ((!array) && (args.length < 3)) {
         mp.error("Macro New: Missing value");
         return "";
      }// if

      final String className = m.group(1);

      int count = array ? (args.length - 2) : 1;
      int length = count;
      if ((m.group(3) != null) && (m.group(3).length() > 0)) {
         length = Integer.decode(m.group(3)).intValue();
      }// if
      if (count > length) {
         mp.warning("Macro New: too many parameters");
         count = length;
      }

      try {

         if ("int".equals(className)) {
            if (array) {
               int[] ints = new int[length];
               for (int i = 0; i < count; i++)
                  ints[i] = Integer.decode(args[i + 2]);
               return mgr.addObject(ints);
            } else {
               return mgr.addObject(Integer.decode(args[2]));
            }
         }
         if ("long".equals(className)) {
            if (array) {
               long[] longs = new long[length];
               for (int i = 0; i < count; i++)
                  longs[i] = Long.decode(args[i + 2]);
               return mgr.addObject(longs);
            } else {
               return mgr.addObject(Long.decode(args[2]));
            }
         }
         if ("short".equals(className)) {
            if (array) {
               short[] shorts = new short[length];
               for (int i = 0; i < count; i++)
                  shorts[i] = Short.decode(args[i + 2]).shortValue();
               return mgr.addObject(shorts);
            } else {
               return mgr.addObject(Short.decode(args[2]));
            }
         }
         if ("byte".equals(className)) {
            if (array) {
               byte[] bytes = new byte[length];
               for (int i = 0; i < count; i++)
                  bytes[i] = Byte.decode(args[i + 2]);
               return mgr.addObject(bytes);
            } else {
               return mgr.addObject(Byte.decode(args[2]));
            }
         }
         if ("float".equals(className)) {
            if (array) {
               float[] floats = new float[length];
               for (int i = 0; i < count; i++)
                  floats[i] = Float.valueOf(args[i + 2]);
               return mgr.addObject(floats);
            } else {
               return mgr.addObject(Float.valueOf(args[2]));
            }
         }
         if ("double".equals(className)) {
            if (array) {
               double[] doubles = new double[length];
               for (int i = 0; i < count; i++)
                  doubles[i] = Double.valueOf(args[i + 2]);
               return mgr.addObject(doubles);
            } else {
               return mgr.addObject(Double.valueOf(args[2]));
            }
         }

         if ("char".equals(className)) {
            if (array) {
               char[] chars = new char[length];
               for (int i = 0; i < count; i++)
                  chars[i] = decodeChar(args[i + 2]).charValue();
               return mgr.addObject(chars);
            } else {
               return mgr.addObject(decodeChar(args[2]));
            }
         }
         if ("boolean".equals(className)) {
            if (array) {
               boolean[] bools = new boolean[length];
               for (int i = 0; i < count; i++)
                  bools[i] = decodeBoolean(args[i + 2]);
               return mgr.addObject(bools);
            } else {
               return mgr.addObject(Boolean.valueOf(decodeBoolean(args[2])));
            }
         }
         Class ac = Class.forName(className);
         Object result = Array.newInstance(ac, length);
         for (int i = 0; i < count; i++)
            Array.set(result, i, mgr.getObject(args[i + 2]));
         return mgr.addObject(result);

      } catch (IndexOutOfBoundsException ex) {
         mp.error("Macro Invoke: Invalid identifier: " + args[1]);
      } catch (ClassNotFoundException ex) {
         mp.error("Macro New: Invalid class name: " + args[1]);
      } catch (NumberFormatException ex) {
         mp.error("Macro New: Invalid value " + args[2] + " for " + args[1]);
      }

      return "";

   }

}
