package pmp.macro.lex;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Decodes C style escape sequence. Invalid escape sequence is ignored and
 * returned unchanged.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class CEscDecoder extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 1;

    @Override
    public String run(Macroprocessor mp, String[] args)
    {

        String str = args[1];

        if ((str.length() < 2) || (str.charAt(0) != '\\')) { return str; } // if

        char result;
        int esclength = 0;
        char type = Character.toLowerCase(str.charAt(1));

        if ((type == 'x')) { // hexa
            if (str.length() < 3) {
                return str;
            }
            else if (str.length() < 4) {
                esclength = 3;
            }
            else {
                esclength = 4;
            }
            result = (char) Integer.parseInt(str.substring(2, esclength + 2),
                    16);
        } // if

        else if ((type == 'u')) { // UNICODE
            if (str.length() < 6) { return str; }
            esclength = 4;
            result = (char) Integer.parseInt(str.substring(2, esclength + 2),
                    16);
        }

        else if (Tools.isDigit(type, 8)) { // OCTAL
            esclength = 0;
            if (Tools.isDigit(str.charAt(2), 8)) {
                esclength = 1;
                if (Tools.isDigit(str.charAt(3), 8)) {
                    esclength = 2;
                } // if
            } // if
            int code = Integer.parseInt(str.substring(1, esclength + 2), 8);
            if (code <= 0377) {
                result = (char) code;
            }
            else {
                return str;
            }
        } // if

        else { // OTHER
            switch (str.charAt(1)) {
            case 'b':
                result = '\u0008';
                break;
            case 'n':
                result = '\n';
                break;
            case 'r':
                result = '\r';
                break;
            case 'f':
                result = '\u000C';
                break;
            case 't':
                result = '\u0009';
                break;
            case 'a':
                result = '\u0007';
                break;
            case '\\':
                result = '\\';
                break;
            case '"':
                result = '\"';
                break;
            case '\'':
                result = '\u002C';
                break;
            default:
                return str;
            } // switch

        } // if

        return (result + str.substring(2 + esclength));

    }

}
