package pmp.macro.lex;

import java.util.HashMap;
import java.util.Map;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Converts HTML entity to string.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * <p>
 * Company:
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 * @todo add all HTML entities
 */

public class HTMLEntityDecoder extends AbstractMacro {

    private Map<String, String> entities = new HashMap<String, String>(32);

    public HTMLEntityDecoder()
    {
        entities.put("apos", "\'");
        entities.put("quot", "\"");
        entities.put("amp", "&");
        entities.put("lt", "<");
        entities.put("gt", ">");
    }

    public static final int PARAMS_REQUIRED = 1;

    @Override
    public String run(Macroprocessor mp, String[] args)
    {
        String ent = args[1];
        if ("".equals(ent) || (ent.charAt(0) != '&')
                || (ent.charAt(ent.length() - 1) != ';')) {
            mp.warning("HTMLEntity: Invalid parameter: " + ent);
            return "";
        } // if
        ent = ent.substring(1, ent.length() - 1);
        if (ent.charAt(0) == '#') {
            try {
                if ((ent.charAt(1) == 'x') || (ent.charAt(1) == 'X')) {
                    int i = Integer.parseInt(ent.substring(2), 16);
                    return Character.toString((char) i);
                } // if
                else {
                    int i = Integer.parseInt(ent.substring(1), 10);
                    return Character.toString((char) i);
                }
            } catch (NumberFormatException ex) {
                mp.warning("Invalid HTML entity: " + args[1]);
                return "";
            }
        }
        else if (entities.containsKey(ent)) {
            return entities.get(ent);
        }
        else {
            mp.warning("Unknown HTML entity: " + args[1]);
            return "";
        }
    }

}
