package pmp.macro.lex;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Converts special characters to XML/HTML entities.
 * <p>
 * <b>Recognized special characters:</b>
 * </p>
 * <ul>
 * <li>'&amp;' becomes '&amp;amp;' </li>
 * <li>'"' becomes '&amp;quot;' when <code>mode != ENT_NOQUOTES</code></li>
 * <li>''' becomes '&amp;#039;' only when <code> mode = ENT_QUOTES</code></li>
 * <li>'&lt;' becomes '&aml;lt;' </li>
 * <li> '&gt;' becomes '&amp;gt;'</li>
 * </ul>
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class HTMLSpecialChars extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 2;

    /** Translate only double quotes. */
    public static final String ENT_COMPAT = "ENT_COMPAT";

    /** Do not translate single or double quotes. */
    public static final String ENT_NOQUOTES = "ENT_NOQUOTES";

    /** Translate both single and double quotes. */
    public static final String ENT_QUOTES = "ENT_QUOTES";

    private static final int MODE_ENT_COMPAT = 0;

    private static final int MODE_ENT_NOQUOTES = 1;

    private static final int MODE_ENT_QUOTES = 2;

    @Override
    public String run(Macroprocessor mp, String[] args)
    {
        int mode = MODE_ENT_COMPAT;
        if (ENT_NOQUOTES.equalsIgnoreCase(args[2])) {
            mode = MODE_ENT_NOQUOTES;
        }
        else if (ENT_QUOTES.equalsIgnoreCase(args[2])) {
            mode = MODE_ENT_QUOTES;
        }

        StringBuilder result = new StringBuilder(args[1].length() * 2);
        char c;

        for (int i = 0; i < args[1].length(); i++) {
            c = args[1].charAt(i);
            switch (c) {

            case '&':
                result.append("&amp;");
                break;

            case '\"':
                if (mode != MODE_ENT_NOQUOTES) {
                    result.append("&quot;");
                }
                else {
                    result.append(c);
                }
                break;

            case '\'':
                if (mode == MODE_ENT_QUOTES) {
                    result.append("&#39;");
                }
                else {
                    result.append(c);
                }
                break;

            case '<':
                result.append("&lt;");
                break;

            case '>':
                result.append("&gt;");
                break;

            default:
                result.append(c);

            } // switch
        } // for

        return result.toString();
    }

}
