package pmp.macro.lex;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Decodes Java style escape sequence. Invalid escape sequence is ignored and
 * returned unchanged. Remaining characters from given string are also returned.
 * It allows using uncomplicated pattern such like: <br />
 * <code>\\.[0-9a-fA-F]{0,4}</code> <br />
 * instead of:<br />
 * <code>\\([^u0-7]|(u[0-9a-fA-F]{4})|([0-3]?[0-7]{1,2}))</code>
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class JavaEscDecoder extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 1;

    @Override
    public String run(Macroprocessor mp, String[] args)
    {

        String str = args[1];
        if ((str.length() < 2) || (str.charAt(0) != '\\')) { return str; } // if

        char result;
        int esclength = 0;

        if (Tools.isDigit(str.charAt(1), 8)) { // octal
            esclength = 2;
            if (Tools.isDigit(str.charAt(2), 8)) {
                esclength = 3;
                if (Tools.isDigit(str.charAt(3), 8)) {
                    esclength = 4;
                } // if
            } // if
            int code = Integer.parseInt(str.substring(1, esclength), 8);
            if (code <= 0377) {
                result = (char) code;
            }
            else {
                return str;
            }
        } // if

        else if (str.charAt(1) == 'u') { // UNICODE
            if (str.length() < 6) { return str; }
            result = (char) Integer.parseInt(str.substring(2, 6), 16);
            esclength = 6;
        } // if

        else { // / OTHER
            esclength = 2;
            switch (str.charAt(1)) {
            case 'b':
                result = '\b';
                break;
            case 'r':
                result = '\r';
                break;
            case 'f':
                result = '\f';
                break;
            case 't':
                result = '\t';
                break;
            case 'n':
                result = '\n';
                break;
            case '\\':
                result = '\\';
                break;
            case '\"':
                result = '\"';
                break;
            case '\'':
                result = '\'';
                break;
            default:
                return str;
            } // switch

        } // if

        return (result + str.substring(esclength));

    }

}
