package pmp.macro.lex;

/**
 * Tools for this package.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

final class Tools {

    /**
     * Instances not required.
     */
    private Tools()
    {
        // Instances not required.
    }

    /**
     * Converts given parameter to C++/Java style string.
     * 
     * @param input
     *            string to be converted
     * @return encoded string
     */
    public static String convertToString(String input)
    {
        StringBuilder result = new StringBuilder(input.length() * 2);
        result.append('\"');

        for (int i = 0; i < input.length(); i++) {
            if (input.charAt(i) == '\"') {
                result.append("\\\"");
            } // if
            else if (input.charAt(i) == '\\') {
                result.append("\\\\");
            } // if
            else {
                result.append(input.charAt(i));
            }
        } // for
        result.append('\"');

        return result.toString();
    }

    /**
     * Returns <code>true</code> if given character is digit in specified
     * radix. Radix must be in range 2 and 36.
     * 
     * @param ch
     *            character
     * @param radix
     *            radix to be used. If greater then 10, first
     *            <code>radix-10</code> letters of alphabet will be considered
     *            to be digits (case-insensitively).
     * @return <code>true</code> if given char was recognized as legal digit
     *         for given radix
     */
    public static boolean isDigit(char ch, int radix)
    {
        if (radix <= 10) {
            return ((ch >= '0') && (ch < ('0' + radix)));
        }
        else {
            radix -= 10;
            return (((ch >= '0') && (ch <= ('9')))
                    || ((ch >= 'a') && (ch < ('a' + radix))) || ((ch >= 'A') && (ch < ('A' + radix))));
        }
    }

}
