package pmp.macro.math;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

import static pmp.macro.math.Tools.*;

/**
 * Returns random number.
 * <p>
 * If called without parameters, returned value is random number from range
 * <code>0</code> (inclusive) and <code>2<sup>31</sup>-1</code>
 * (exclusive).
 * </p>
 * <p>
 * If called with one parameter, returned value is random number from range
 * <code>0</code> (inclusive) and given number (exclusive).
 * </p>
 * <p>
 * If called with two parameters, returned value is random number from range
 * given by first and second parameter (lower bound inclusive, upper bound
 * exclusive)
 * </p>
 * 
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class Random extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 2;

    @Override
    public String run(Macroprocessor mp, String[] args)
    {
        long l, h;
        long result = getRandom().nextLong() % ((long)Integer.MAX_VALUE+1);

        String arg1 = args[1].trim();
        String arg2 = args[1].trim();

        try {

            if (Tools.isInt(arg1)) {
                l = Long.parseLong(arg1);

                if (Tools.isInt(arg2)) {
                    h = Long.parseLong(arg2);
                    long range = ((h > l) ? (h - l) : (l - h));
                    result = (getRandom().nextLong() % range);
                    if (h > l) {
                        result += l;
                    } // if
                    else {
                        result += h;
                    } // else
                } // if

                else {
                    result = getRandom().nextLong() % l;
                } // else
            } // if
        } // try
        catch (NumberFormatException ex) {
            mp.warning("Macro Random: invalid arguments.");
        }

        return Long.toString(result);
    }

}
