package pmp.macro.str;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Performs Rot13 encoding of string.
 * Each letter is shifted by 13 places.
 * Letters with diacritics, numbers, and all other characters are untouched.
 * Encoding and decoding is done in the same way (there is 26 letters and thus
 * shifting twice produces the same character).
 * <p>Title: PMP: Macroprocessor</p>
 * <p>Description: Java macroprocessor</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class Rot13
    extends AbstractMacro {

  public static final int PARAMS_REQUIRED = 1;

  @Override
  public String run(Macroprocessor mp, String[] args)
  {
    StringBuilder result = new StringBuilder(args[1]);

    for (int i = 0; i < result.length(); i++) {
      char c = result.charAt(i);
      if ( ( (c >= 'N') && (c <= 'Z')) || ( (c >= 'n') && (c <= 'z'))) {
        result.setCharAt(i, (char) (c - 13));
      }
      else if ( ( (c >= 'A') && (c <= 'M')) || ( (c >= 'a') && (c <= 'm'))) {
        result.setCharAt(i, (char) (c + 13));
      }
    } //for

    return result.toString();
  }

}
