package pmp.macro.str;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Returns substring of given string. The first argument contains given string,
 * the second is index of first character (zero-based) to be copied and the
 * third is maximal number of characters to be copied. Length parameter may be
 * omitted - then all characters up to end of string will be copied
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class SubStr extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 3;

    @Override
    public String run(Macroprocessor mp, String[] args)
    {
        String str = args[1];
        int start = 0;
        int end = str.length();
        try {

            if (!"".equals(args[2])) {
                start = Integer.parseInt(args[2]);

                if (!"".equals(args[3])) {
                    end = start + Integer.parseInt(args[3]);
                    if (end > str.length()) {
                        mp
                                .notice("Macro SubStr: Not enough characters to be extracted.");
                        end = str.length();
                    } // if
                } // if

            } // if

        } // try
        catch (NumberFormatException ex) {
            mp.warning("SubStr macro: invalid number format " + args[2] + ", "
                    + args[3]);

        } // catch
        try {
            return str.substring(start, end);
        } // try
        catch (IndexOutOfBoundsException ex) {
            mp.warning("SubStr macro: invalid range " + start + ", " + end
                    + "for string: " + args[1]);
        } // catch

        return str;
    }

}
