package pmp.macro.str;

import java.util.Arrays;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Strips whitespace from the beginning and/or end of a string.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class Trim extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 2;

    public static final String BOTH = "BOTH";

    public static final String LEADING = "LEADING";

    public static final String TRAILING = "TRAILING";

    private static final int MODE_BOTH = 0;

    private static final int MODE_LEADING = 1;

    private static final int MODE_TRAILING = 2;

    @Override
    public String run(Macroprocessor mp, String[] params)
    {
        int mode = MODE_BOTH;
        if (LEADING.equalsIgnoreCase(params[2])) {
            mode = MODE_LEADING;
        }
        else if (TRAILING.equalsIgnoreCase(params[2])) {
            mode = MODE_TRAILING;
        }

        String str = params[1];

        int start = 0;
        int end = str.length() - 1;

        if (mode != MODE_TRAILING) { // for modes LEADING & BOTH
            while ((start < str.length())
                    && (Arrays.binarySearch(Tools.getSpaceCharacters(), str
                            .charAt(start)) >= 0)) {
                start++;
            } // while
        } // if
        if (mode != MODE_LEADING) { // for modes TRAILING & BOTH
            while ((end >= 0)
                    && (Arrays.binarySearch(Tools.getSpaceCharacters(), str
                            .charAt(end)) >= 0)) {
                end--;
            } // while
        } // if

        if (start <= end) {
            return str.substring(start, end + 1); // end+1, as we want char at
            // str[end]
        }
        else {
            return "";
        }
    }
}
