package pmp.macro.str;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Converts parameter to upper case.
 * 
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 * @todo modes !!!
 */

public class UpperCase extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 2;

    /**
     * Ignore locale settings and modify only ASCII characters
     * <code>[a-zA-Z]</code>.
     */
    public static final String UC_ASCII = "ASCII";

    /**
     * Change all letters to upper case. This is default mode.
     */
    public static final String UC_ALL = "ALL";

    /**
     * Change to upper case only the first letters in words.
     */
    public static final String UC_WORDS = "WORDS";

    /**
     * Change only the first letter to upper case.
     */
    public static final String UC_FIRST = "FIRST";

    @Override
    public String run(Macroprocessor mp, String[] params)
    {
        final int ASCII = 1;
        final int FIRST = 4;
        final int WORDS = 2 | FIRST;
        final int ALL = 8 | FIRST | WORDS;

        int mode = 0;

        for (int i = 2; i < params.length; i++) {
            // String p = params[i].trim().toUpperCase();
            if (UC_ASCII.equals(params[i])) {
                mode |= ASCII;
            }
            else if (UC_ALL.equals(params[i])) {
                mode |= ALL;
            }
            else if (UC_FIRST.equals(params[i])) {
                mode |= FIRST;
            }
            else if (UC_WORDS.equals(params[i])) {
                mode |= WORDS;
            }
        } // for

        return params[1].toUpperCase(mp.getLocale());
    }

}
