/**
 * 
 */
package pmp.macro.tools;

import static pmp.ConfigDirectives.INPUT_ENCODING;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;

import pmp.AbstractMacro;
import pmp.FatalParserException;
import pmp.MacroList;
import pmp.Macroprocessor;

/**
 * Executes given command.
 * Program name is specified by the first parameter. 
 * Remaining parameters are passed as command-line arguments.
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class Exec extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 1;

    @Override
    public String run(Macroprocessor mp, String[] args)
    {
        String[] cmd = new String[args.length - 1];
        System.arraycopy(args, 1, cmd, 0, args.length - 1);

        try {
            Process p = Runtime.getRuntime().exec(cmd);

            MacroList ml = mp.getMPMacros();
            Reader r;
            if (ml.isDirectiveDefined(INPUT_ENCODING)) {
                String enc = ml.getString(INPUT_ENCODING);
                try {
                    r = new InputStreamReader(p.getInputStream(), enc);
                } catch (UnsupportedEncodingException e) {
                    mp.warning("Macro Exec: unsupported encoding " + enc);
                    r = new InputStreamReader(p.getInputStream());
                }
            }//if
            else {
                r = new InputStreamReader(p.getInputStream());
            }//else
            BufferedReader in = new BufferedReader(r);
            StringBuilder buffer = new StringBuilder(512);

            String currentLine = null; 
            while ((currentLine = in.readLine()) != null)
                buffer.append(currentLine);
            
            mp.getMPIQ().pushFront(buffer);
            
        } catch (IOException e) {
            throw new FatalParserException("Macro Exec failed: " + e.toString());
        }

        return "";
    }

}
