package pmp.macro.tools;

import java.io.IOException;

import pmp.AbstractMacro;
import pmp.Macroprocessor;

/**
 * Pauses for given amount of time or until key is pressed if no value is given.
 * First parameter contains time to wait in milliseconds.
 * <p>
 * The second parameter specifies mode of interaction with standard input. See
 * description of fields of this class for details.
 * </p>
 * <p>
 * Note: Waiting for keypress doesn't work when invoked from Ant.
 * </p>
 * <p>
 * If given value is greater than 1000, <code>System.gc()</code> is invoked.
 * </p>
 * <p>
 * Title: PMP: Macroprocessor
 * </p>
 * <p>
 * Description: Java macroprocessor
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * 
 * @author Lud\u011Bk Hlav\u00E1\u010Dek
 * @version 1.0
 */

public class Pause extends AbstractMacro {

    public static final int PARAMS_REQUIRED = 2;

    /**
     * Waits for given amount of time and then exits (default mode).
     */
    public static final String SLEEP = "SLEEP";

    /**
     * Waits for given amount of time and then waits for keypress.
     */
    public static final String WAIT = "WAIT";

    /**
     * Waits until elapses given amount of time or until keypress.
     */
    public static final String INTR = "INTR";

    private boolean testKey()
    {
        try {
            if (System.in.available() > 0) {
                while (System.in.available() > 0) {
                    System.in.read();
                } // while
                return true;
            } // if
        } catch (IOException ex) {
            // ignore
        }
        return false;
    }

    @Override
    public String run(Macroprocessor mp, String[] args)
    {
        long start = System.currentTimeMillis();
        long time = Integer.parseInt(args[1]);

        final int MODE_SLEEP = 0;
        final int MODE_WAIT = 1;
        final int MODE_INTR = 2;

        int mode = MODE_SLEEP;

        String m = args[2].trim().toUpperCase();

        if (m.equals(SLEEP)) {
            mode = MODE_SLEEP;
        }
        else if (m.equals(WAIT)) {
            mode = MODE_WAIT;
        }
        else if (m.equals(INTR)) {
            mode = MODE_INTR;
        }

        if ((time > 1000) || (mode == MODE_WAIT)) {
            System.gc();
        }

        long timeslice = 200;

        if (time > 0) {
            final long end = start + time;

            while (System.currentTimeMillis() < end) {
                try {
                    Thread.sleep(timeslice);
                } catch (InterruptedException ex) {
                    // ignore
                }

                if ((mode == MODE_INTR) && testKey()) {
                    break;
                } // if
            }
        } // if

        if (mode == MODE_WAIT) {
            testKey();

            while (true) {
                try {
                    Thread.sleep(timeslice);
                } // try
                catch (InterruptedException ex) {
                    // ignore
                }
                if (testKey()) {
                    break;
                } // if
            } // while
        } // if

        return "";
    }

}
